--
-- subsoilerActive
-- Specialization for subsoilerActive settings
--
-- @author John Deere 6930
-- @date 26/11/2019
--

--[[XML

    <cultivator> <!-- isSubsoiler="true" no effect by subsoilerActive! -->
		<subsoilerActive ifNoAttachedVehicle="true" >
			<subsoilerAnimation animationName="" subsoilerTime="" />
			<subsoilerInfoText activeText="" deactiveText="" detachText=""/>
		</subsoilerActive>
    </cultivator>

]]

SubsoilerActive = {};
SubsoilerActive.modDir = g_currentModDirectory;
SubsoilerActive.MOD_NAME = g_currentModName;

function SubsoilerActive.prerequisitesPresent(specializations)
    return  SpecializationUtil.hasSpecialization(Cultivator, specializations) and SpecializationUtil.hasSpecialization(AttacherJoints, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function SubsoilerActive.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "processSubsoilerActiveArea", 		SubsoilerActive.processSubsoilerActiveArea);
end;

function SubsoilerActive.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", SubsoilerActive);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", SubsoilerActive);
	SpecializationUtil.registerEventListener(vehicleType, "onDraw", SubsoilerActive);
end;

function SubsoilerActive:onLoad(savegame)
    self.spec_subsoilerActive = self[string.format("spec_%s.subsoilerActive", SubsoilerActive.MOD_NAME)];
    local spec = self.spec_subsoilerActive;	
	spec.subsoilerIsForcedActive = false;
	
	spec.ifNoAttachedVehicle = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.cultivator.subsoilerActive#ifNoAttachedVehicle"), false);		
	if hasXMLProperty(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerAnimation") then
		local animationName = getXMLString(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerAnimation#animationName");
		if animationName ~= nil then
			spec.subsoilerAnimation = animationName;
			spec.subsoilerAnimationTime = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerAnimation#subsoilerTime"), 1.0);
		end;		
	end;	
	if hasXMLProperty(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerInfoText") then
		local infoTextActive = getXMLString(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerInfoText#activeText");
		if infoTextActive ~= nil then
			spec.infoTextActive = infoTextActive;
		end;
		local infoTextDeactive = getXMLString(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerInfoText#deactiveText");
		if infoTextDeactive ~= nil then
			spec.infoTextDeactive = infoTextDeactive;
		end;
		local infoTextDetach = getXMLString(self.xmlFile, "vehicle.cultivator.subsoilerActive.subsoilerInfoText#detachText");
		if infoTextDetach ~= nil then
			spec.infoTextDetach = infoTextDetach;
		end;
	end;
end;

function SubsoilerActive:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    local spec = self.spec_subsoilerActive;
	if self:getIsActive() then
		spec.subsoilerIsForcedActive = true;
		if spec.ifNoAttachedVehicle then	
			local attachedImplements = self:getAttachedImplements();
			if #attachedImplements > 0 then
				spec.subsoilerIsForcedActive = false;
			end;
		end;
		if spec.subsoilerAnimation ~= nil then
			if spec.subsoilerIsForcedActive then
				local animTime = self:getAnimationTime(spec.subsoilerAnimation);
				spec.subsoilerIsForcedActive = animTime >= spec.subsoilerAnimationTime;
			end;
		end;
	end;
end;

function SubsoilerActive:onDraw(isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    local spec = self.spec_subsoilerActive;
	if spec.infoTextActive ~= nil and spec.infoTextDeactive ~= nil then
		if spec.subsoilerIsForcedActive then
			g_currentMission:addExtraPrintText(g_i18n:getText(spec.infoTextActive));
		else
			if spec.ifNoAttachedVehicle then	
				local attachedImplements = self:getAttachedImplements();
				if #attachedImplements > 0 then
					g_currentMission:addExtraPrintText(g_i18n:getText(spec.infoTextDetach));
				else
					g_currentMission:addExtraPrintText(g_i18n:getText(spec.infoTextDeactive));
				end;
			else
				g_currentMission:addExtraPrintText(g_i18n:getText(spec.infoTextDeactive));			
			end;
		end;
	end;
end;

function SubsoilerActive:processSubsoilerActiveArea(workArea, dt)
    local spec = self.spec_cultivator;
    local xs,_,zs = getWorldTranslation(workArea.start);
    local xw,_,zw = getWorldTranslation(workArea.width);
    local xh,_,zh = getWorldTranslation(workArea.height);
    local params = spec.workAreaParameters;
    local realArea, area = FSDensityMapUtil.updateCultivatorArea(xs,zs, xw,zw, xh,zh, not params.limitToField, not params.limitGrassDestructionToField, params.angle, nil);
    params.lastChangedArea = params.lastChangedArea + realArea;
    params.lastStatsArea = params.lastStatsArea + realArea;
    params.lastTotalArea = params.lastTotalArea + area;
		
    if spec.isSubsoiler or self.spec_subsoilerActive.subsoilerIsForcedActive then
        FSDensityMapUtil.updateSubsoilerArea(xs,zs, xw,zw, xh,zh);
    end;
	
    FSDensityMapUtil.eraseTireTrack(xs,zs, xw,zw, xh,zh)
    spec.isWorking = self:getLastSpeed() > 0.5;
    return realArea, area;
end;