--
-- VariablePowerMultiplier
-- Specialization for VariablePowerMultiplier settings
--
-- @author John Deere 6930
-- @date 20/04/2020
--

--[[XML

    <powerConsumer ... >
		<variablePowerMultiplier animationName="STRING" startMultiplier="FLOAT" endMultiplier="FLOAT"/>
	</powerConsumer>

]]

VariablePowerMultiplier = {};
VariablePowerMultiplier.modDir = g_currentModDirectory;
VariablePowerMultiplier.MOD_NAME = g_currentModName;

function VariablePowerMultiplier.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(GroundReference, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function VariablePowerMultiplier.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", VariablePowerMultiplier);
end;

function VariablePowerMultiplier.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getPowerMultiplier", VariablePowerMultiplier.getPowerMultiplier)
end

function VariablePowerMultiplier:onLoad(savegame)
    self.spec_variablePowerMultiplier = self[string.format("spec_%s.variablePowerMultiplier", VariablePowerMultiplier.MOD_NAME)];
    local spec = self.spec_variablePowerMultiplier;	
	
	if hasXMLProperty(self.xmlFile, "vehicle.powerConsumer.variablePowerMultiplier") then	
		local animationName = getXMLString(self.xmlFile, "vehicle.powerConsumer.variablePowerMultiplier#animationName");
		if animationName ~= nil then
			spec.animationName = animationName;
		end;
		spec.startMultiplier = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.powerConsumer.variablePowerMultiplier#startMultiplier"), 0.0)
		local endMultiplier = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.powerConsumer.variablePowerMultiplier#endMultiplier"), 1.0)
		spec.multiplierRange = endMultiplier-spec.startMultiplier;
	end;
end;

function VariablePowerMultiplier:getPowerMultiplier(superFunc)
	local spec = self.spec_variablePowerMultiplier;	
	local multiplier = superFunc(self)
	if spec.animationName ~= nil then
		local animTime = self:getAnimationTime(spec.animationName);		
		multiplier = multiplier*(spec.startMultiplier+spec.multiplierRange*animTime);
	end;
    return multiplier;
end